
use builtin;
use str;

set edit:completion:arg-completer[rustup] = {|@words|
    fn spaces {|n|
        builtin:repeat $n ' ' | str:join ''
    }
    fn cand {|text desc|
        edit:complex-candidate $text &display=$text' '(spaces (- 14 (wcswidth $text)))$desc
    }
    var command = 'rustup'
    for word $words[1..-1] {
        if (str:has-prefix $word '-') {
            break
        }
        set command = $command';'$word
    }
    var completions = [
        &'rustup'= {
            cand -v 'Enable verbose output'
            cand --verbose 'Enable verbose output'
            cand -q 'Disable progress output'
            cand --quiet 'Disable progress output'
            cand -h 'Print help'
            cand --help 'Print help'
            cand -V 'Print version'
            cand --version 'Print version'
            cand dump-testament 'Dump information about the build'
            cand show 'Show the active and installed toolchains or profiles'
            cand install 'Update Rust toolchains'
            cand uninstall 'Uninstall Rust toolchains'
            cand update 'Update Rust toolchains and rustup'
            cand check 'Check for updates to Rust toolchains and rustup'
            cand default 'Set the default toolchain'
            cand toolchain 'Modify or query the installed toolchains'
            cand target 'Modify a toolchain''s supported targets'
            cand component 'Modify a toolchain''s installed components'
            cand override 'Modify toolchain overrides for directories'
            cand run 'Run a command with an environment configured for a given toolchain'
            cand which 'Display which binary will be run for a given command'
            cand doc 'Open the documentation for the current toolchain'
            cand man 'View the man page for a given command'
            cand self 'Modify the rustup installation'
            cand set 'Alter rustup settings'
            cand completions 'Generate tab-completion scripts for your shell'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;dump-testament'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;show'= {
            cand -v 'Enable verbose output with rustc information for all installed toolchains'
            cand --verbose 'Enable verbose output with rustc information for all installed toolchains'
            cand -h 'Print help'
            cand --help 'Print help'
            cand active-toolchain 'Show the active toolchain'
            cand home 'Display the computed value of RUSTUP_HOME'
            cand profile 'Show the default profile used for the `rustup install` command'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;show;active-toolchain'= {
            cand -v 'Enable verbose output with rustc information'
            cand --verbose 'Enable verbose output with rustc information'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;show;home'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;show;profile'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;show;help'= {
            cand active-toolchain 'Show the active toolchain'
            cand home 'Display the computed value of RUSTUP_HOME'
            cand profile 'Show the default profile used for the `rustup install` command'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;show;help;active-toolchain'= {
        }
        &'rustup;show;help;home'= {
        }
        &'rustup;show;help;profile'= {
        }
        &'rustup;show;help;help'= {
        }
        &'rustup;install'= {
            cand --profile 'profile'
            cand --no-self-update 'Don''t perform self-update when running the `rustup install` command'
            cand --force 'Force an update, even if some components are missing'
            cand --force-non-host 'Install toolchains that require an emulator. See https://github.com/rust-lang/rustup/wiki/Non-host-toolchains'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;uninstall'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;update'= {
            cand --no-self-update 'Don''t perform self update when running the `rustup update` command'
            cand --force 'Force an update, even if some components are missing'
            cand --force-non-host 'Install toolchains that require an emulator. See https://github.com/rust-lang/rustup/wiki/Non-host-toolchains'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;check'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;default'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;toolchain'= {
            cand -h 'Print help'
            cand --help 'Print help'
            cand list 'List installed toolchains'
            cand install 'Install or update a given toolchain'
            cand uninstall 'Uninstall a toolchain'
            cand link 'Create a custom toolchain by symlinking to a directory'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;toolchain;list'= {
            cand -v 'Enable verbose output with toolchain information'
            cand --verbose 'Enable verbose output with toolchain information'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;toolchain;install'= {
            cand --profile 'profile'
            cand -c 'Add specific components on installation'
            cand --component 'Add specific components on installation'
            cand -t 'Add specific targets on installation'
            cand --target 'Add specific targets on installation'
            cand --no-self-update 'Don''t perform self update when running the`rustup toolchain install` command'
            cand --force 'Force an update, even if some components are missing'
            cand --allow-downgrade 'Allow rustup to downgrade the toolchain to satisfy your component choice'
            cand --force-non-host 'Install toolchains that require an emulator. See https://github.com/rust-lang/rustup/wiki/Non-host-toolchains'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;toolchain;uninstall'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;toolchain;link'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;toolchain;help'= {
            cand list 'List installed toolchains'
            cand install 'Install or update a given toolchain'
            cand uninstall 'Uninstall a toolchain'
            cand link 'Create a custom toolchain by symlinking to a directory'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;toolchain;help;list'= {
        }
        &'rustup;toolchain;help;install'= {
        }
        &'rustup;toolchain;help;uninstall'= {
        }
        &'rustup;toolchain;help;link'= {
        }
        &'rustup;toolchain;help;help'= {
        }
        &'rustup;target'= {
            cand -h 'Print help'
            cand --help 'Print help'
            cand list 'List installed and available targets'
            cand add 'Add a target to a Rust toolchain'
            cand remove 'Remove a target from a Rust toolchain'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;target;list'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --installed 'List only installed targets'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;target;add'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;target;remove'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;target;help'= {
            cand list 'List installed and available targets'
            cand add 'Add a target to a Rust toolchain'
            cand remove 'Remove a target from a Rust toolchain'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;target;help;list'= {
        }
        &'rustup;target;help;add'= {
        }
        &'rustup;target;help;remove'= {
        }
        &'rustup;target;help;help'= {
        }
        &'rustup;component'= {
            cand -h 'Print help'
            cand --help 'Print help'
            cand list 'List installed and available components'
            cand add 'Add a component to a Rust toolchain'
            cand remove 'Remove a component from a Rust toolchain'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;component;list'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --installed 'List only installed components'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;component;add'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --target 'target'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;component;remove'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --target 'target'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;component;help'= {
            cand list 'List installed and available components'
            cand add 'Add a component to a Rust toolchain'
            cand remove 'Remove a component from a Rust toolchain'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;component;help;list'= {
        }
        &'rustup;component;help;add'= {
        }
        &'rustup;component;help;remove'= {
        }
        &'rustup;component;help;help'= {
        }
        &'rustup;override'= {
            cand -h 'Print help'
            cand --help 'Print help'
            cand list 'List directory toolchain overrides'
            cand set 'Set the override toolchain for a directory'
            cand unset 'Remove the override toolchain for a directory'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;override;list'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;override;set'= {
            cand --path 'Path to the directory'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;override;unset'= {
            cand --path 'Path to the directory'
            cand --nonexistent 'Remove override toolchain for all nonexistent directories'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;override;help'= {
            cand list 'List directory toolchain overrides'
            cand set 'Set the override toolchain for a directory'
            cand unset 'Remove the override toolchain for a directory'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;override;help;list'= {
        }
        &'rustup;override;help;set'= {
        }
        &'rustup;override;help;unset'= {
        }
        &'rustup;override;help;help'= {
        }
        &'rustup;run'= {
            cand --install 'Install the requested toolchain if needed'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;which'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', ''1.8.0'', or a custom toolchain name. For more information see `rustup help toolchain`'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;doc'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --path 'Only print the path to the documentation'
            cand --alloc 'The Rust core allocation and collections library'
            cand --book 'The Rust Programming Language book'
            cand --cargo 'The Cargo Book'
            cand --core 'The Rust Core Library'
            cand --edition-guide 'The Rust Edition Guide'
            cand --nomicon 'The Dark Arts of Advanced and Unsafe Rust Programming'
            cand --proc_macro 'A support library for macro authors when defining new macros'
            cand --reference 'The Rust Reference'
            cand --rust-by-example 'A collection of runnable examples that illustrate various Rust concepts and standard libraries'
            cand --rustc 'The compiler for the Rust programming language'
            cand --rustdoc 'Documentation generator for Rust projects'
            cand --std 'Standard library API documentation'
            cand --test 'Support code for rustc''s built in unit-test and micro-benchmarking framework'
            cand --unstable-book 'The Unstable Book'
            cand --embedded-book 'The Embedded Rust Book'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;man'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;self'= {
            cand -h 'Print help'
            cand --help 'Print help'
            cand update 'Download and install updates to rustup'
            cand uninstall 'Uninstall rustup.'
            cand upgrade-data 'Upgrade the internal data format.'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;self;update'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;self;uninstall'= {
            cand -y 'y'
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;self;upgrade-data'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;self;help'= {
            cand update 'Download and install updates to rustup'
            cand uninstall 'Uninstall rustup.'
            cand upgrade-data 'Upgrade the internal data format.'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;self;help;update'= {
        }
        &'rustup;self;help;uninstall'= {
        }
        &'rustup;self;help;upgrade-data'= {
        }
        &'rustup;self;help;help'= {
        }
        &'rustup;set'= {
            cand -h 'Print help'
            cand --help 'Print help'
            cand default-host 'The triple used to identify toolchains when not specified'
            cand profile 'The default components installed with a toolchain'
            cand auto-self-update 'The rustup auto self update mode'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;set;default-host'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;set;profile'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;set;auto-self-update'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;set;help'= {
            cand default-host 'The triple used to identify toolchains when not specified'
            cand profile 'The default components installed with a toolchain'
            cand auto-self-update 'The rustup auto self update mode'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;set;help;default-host'= {
        }
        &'rustup;set;help;profile'= {
        }
        &'rustup;set;help;auto-self-update'= {
        }
        &'rustup;set;help;help'= {
        }
        &'rustup;completions'= {
            cand -h 'Print help'
            cand --help 'Print help'
        }
        &'rustup;help'= {
            cand dump-testament 'Dump information about the build'
            cand show 'Show the active and installed toolchains or profiles'
            cand install 'Update Rust toolchains'
            cand uninstall 'Uninstall Rust toolchains'
            cand update 'Update Rust toolchains and rustup'
            cand check 'Check for updates to Rust toolchains and rustup'
            cand default 'Set the default toolchain'
            cand toolchain 'Modify or query the installed toolchains'
            cand target 'Modify a toolchain''s supported targets'
            cand component 'Modify a toolchain''s installed components'
            cand override 'Modify toolchain overrides for directories'
            cand run 'Run a command with an environment configured for a given toolchain'
            cand which 'Display which binary will be run for a given command'
            cand doc 'Open the documentation for the current toolchain'
            cand man 'View the man page for a given command'
            cand self 'Modify the rustup installation'
            cand set 'Alter rustup settings'
            cand completions 'Generate tab-completion scripts for your shell'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;help;dump-testament'= {
        }
        &'rustup;help;show'= {
            cand active-toolchain 'Show the active toolchain'
            cand home 'Display the computed value of RUSTUP_HOME'
            cand profile 'Show the default profile used for the `rustup install` command'
        }
        &'rustup;help;show;active-toolchain'= {
        }
        &'rustup;help;show;home'= {
        }
        &'rustup;help;show;profile'= {
        }
        &'rustup;help;install'= {
        }
        &'rustup;help;uninstall'= {
        }
        &'rustup;help;update'= {
        }
        &'rustup;help;check'= {
        }
        &'rustup;help;default'= {
        }
        &'rustup;help;toolchain'= {
            cand list 'List installed toolchains'
            cand install 'Install or update a given toolchain'
            cand uninstall 'Uninstall a toolchain'
            cand link 'Create a custom toolchain by symlinking to a directory'
        }
        &'rustup;help;toolchain;list'= {
        }
        &'rustup;help;toolchain;install'= {
        }
        &'rustup;help;toolchain;uninstall'= {
        }
        &'rustup;help;toolchain;link'= {
        }
        &'rustup;help;target'= {
            cand list 'List installed and available targets'
            cand add 'Add a target to a Rust toolchain'
            cand remove 'Remove a target from a Rust toolchain'
        }
        &'rustup;help;target;list'= {
        }
        &'rustup;help;target;add'= {
        }
        &'rustup;help;target;remove'= {
        }
        &'rustup;help;component'= {
            cand list 'List installed and available components'
            cand add 'Add a component to a Rust toolchain'
            cand remove 'Remove a component from a Rust toolchain'
        }
        &'rustup;help;component;list'= {
        }
        &'rustup;help;component;add'= {
        }
        &'rustup;help;component;remove'= {
        }
        &'rustup;help;override'= {
            cand list 'List directory toolchain overrides'
            cand set 'Set the override toolchain for a directory'
            cand unset 'Remove the override toolchain for a directory'
        }
        &'rustup;help;override;list'= {
        }
        &'rustup;help;override;set'= {
        }
        &'rustup;help;override;unset'= {
        }
        &'rustup;help;run'= {
        }
        &'rustup;help;which'= {
        }
        &'rustup;help;doc'= {
        }
        &'rustup;help;man'= {
        }
        &'rustup;help;self'= {
            cand update 'Download and install updates to rustup'
            cand uninstall 'Uninstall rustup.'
            cand upgrade-data 'Upgrade the internal data format.'
        }
        &'rustup;help;self;update'= {
        }
        &'rustup;help;self;uninstall'= {
        }
        &'rustup;help;self;upgrade-data'= {
        }
        &'rustup;help;set'= {
            cand default-host 'The triple used to identify toolchains when not specified'
            cand profile 'The default components installed with a toolchain'
            cand auto-self-update 'The rustup auto self update mode'
        }
        &'rustup;help;set;default-host'= {
        }
        &'rustup;help;set;profile'= {
        }
        &'rustup;help;set;auto-self-update'= {
        }
        &'rustup;help;completions'= {
        }
        &'rustup;help;help'= {
        }
    ]
    $completions[$command]
}
